//
//  HomeViewController.m
//
// ADOBE SYSTEMS INCORPORATED
// Copyright 2014 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the
// terms of the Adobe license agreement accompanying it.  If you have received this file from a
// source other than Adobe, then your use, modification, or distribution of it requires the prior
// written permission of Adobe.
//

#import "HomeViewController.h"
#import "MainViewController.h"
#import "PGViewController.h"
#import "WebViewDelegate.h"

@interface HomeViewController ()
	@property (weak, nonatomic) IBOutlet UISwitch *hideNavBar;
	@property NSMutableArray *historyUrls;
	@property WebViewDelegate *webViewDelegate;
	@property NSString *urlToNavigate;
@end

@implementation HomeViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

	self.webViewDelegate = [[WebViewDelegate alloc] initWithWebView:self.embeddedWebView  withWebViewInterface:self];

	self.embeddedWebView.scrollView.scrollEnabled = false;

	self.navigationController.delegate = self;
	
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSArray *urls = [defaults arrayForKey:@"urls"];
	if (urls != nil)
		self.historyUrls = [NSMutableArray arrayWithArray:urls];
	else
	{
		self.historyUrls = [[NSMutableArray alloc] init];
		[defaults setObject:self.historyUrls forKey:@"urls"];
	}
	
	
	[self.embeddedWebView loadRequest:[NSURLRequest requestWithURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"index" ofType:@"html" inDirectory:@"www/shell"]isDirectory:NO]]];
	
	
	if (self.historyUrls.count > 0)
		self.urlTxtField.text = [self.historyUrls objectAtIndex:0];
	
	
	UITapGestureRecognizer *tapReco = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onTapGesture:)];
	tapReco.cancelsTouchesInView = false;
	[self.view addGestureRecognizer:tapReco];
	
	
}

- (void) onTapGesture:(UITapGestureRecognizer *) sender
{
	[self.view endEditing:true];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)navigationController:(UINavigationController *)navigationController willShowViewController:(UIViewController *)viewController animated:(BOOL)animated
{
	if ([viewController isKindOfClass:[MainViewController class]])
	{
		MainViewController *mainCtrl = (MainViewController*) viewController;

		NSMutableString *urlStr = [NSMutableString stringWithString:self.urlToNavigate];
		
		NSRange range = [urlStr rangeOfString:@"?"];
		if (range.length == 0)
		{
			//not found
			[urlStr appendString:@"?os=ios"];
		} else
		{
			[urlStr appendString:@"&os=ios"];
		}

		NSURLRequest* req = [NSURLRequest requestWithURL:[NSURL URLWithString:urlStr]];
		
		[mainCtrl.webView loadRequest:req];
		
		if (self.hideNavBar.on)
			navigationController.navigationBar.hidden = true;
	}
}

- (id) processFunctionFromJS:(NSString *) name withArgs:(NSArray*) args error:(NSError **) error
{
	if ([name compare:@"getUrls" options:NSCaseInsensitiveSearch] == NSOrderedSame)
	{
//		NSArray *urlArray = [[NSArray alloc] initWithObjects:@"http://www.rediff.com",@"http://www.timesofindia.com", nil];

		NSData *jsonData = [NSJSONSerialization dataWithJSONObject:self.historyUrls options:0 error:nil];
		
		NSString *result = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
		
		return result;
	}
	else if ([name compare:@"openUrl" options:NSCaseInsensitiveSearch] == NSOrderedSame)
	{
		if (args.count > 0)
		{
			self.urlTxtField.text = [args objectAtIndex:0];
			[self addURLToDefaults:self.urlTxtField.text];
			
			
			self.urlToNavigate = [args objectAtIndex:0];
			if (self.urlToNavigate.length == 0)
			{
				[self.webViewDelegate createError:error
				 withCode:-1 withMessage:@"No URL selected"];
				 return nil;
			}

			
		    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"main" bundle:nil];
			
			PGViewController *mainCtrl = [storyboard instantiateViewControllerWithIdentifier:@"PGViewController"];

			[self.navigationController pushViewController:mainCtrl animated:true];
			return nil;
		}
		else
		{
			[self.webViewDelegate createError:error withCode:-1 withMessage:@"Wrong number of arguments passed to openUrl method"];
			return nil;
		}
		
	}
	
	//everything else is error
	NSString *msg = [NSString stringWithFormat:@"Function %@ not found",name];
    [self.webViewDelegate createError:error withCode:-1 withMessage:msg];
	return nil;
}

- (IBAction)onGoBtn:(id)sender {
	self.urlToNavigate = self.urlTxtField.text;
	
	if (self.urlToNavigate.length == 0)
		return;
	
	[self addURLToDefaults:self.urlToNavigate];

	
	NSDictionary *dict = @{@"url":self.urlToNavigate};
		
	[self.webViewDelegate callJSFunction:@"appendUrlToList" withArgs:dict];
}

- (void) addURLToDefaults:(NSString *) url
{
	int maxLimit = 10;
		
	int index = [self.historyUrls indexOfObject:url];
	if (index != NSNotFound)
		[self.historyUrls removeObjectAtIndex:index];
				
	for (int i = [self.historyUrls count]-1; i >= 0 ; i--)
	{
		if (i >= maxLimit)
			continue;
		
		[self.historyUrls setObject:[self.historyUrls objectAtIndex:i] atIndexedSubscript:i+1];
	}
		
	[self.historyUrls setObject:url atIndexedSubscript:0];
	
	NSString *domainName = [[NSBundle mainBundle] bundleIdentifier];
	[[NSUserDefaults standardUserDefaults] removePersistentDomainForName:domainName];
	
	[[NSUserDefaults standardUserDefaults] setObject:self.historyUrls forKey:@"urls"];
	
}
@end
